/*
     Object Oriented Programming using C++,
     Solution Manual and Applied Examples.
	    written by Dr. Farschad Torabi
     K. N. Toosi University of Technology  
	           September, 2013

                Chapter 26
      Code written by: Syawash Azizi
*/

#include<iostream>
#include<math.h>
using namespace std;

class Fin
{
public: 
  virtual void heat()=0;
  virtual void Temp()=0;
  double k,t,w,h,T_b,T_be,x,L,P,Ac,teta_b,teta_L,T,q,Co,M,m,T_L;
  void ReadData()
  {
    cout<<"Please Enter Fin length ........... L    [m]      : "; cin>>L;
    cout<<"Please Enter Fin thickness ........ t    [m]      : "; cin>>t;
    cout<<"Please Enter Fin height ........... w    [m]      : "; cin>>w;
    cout<<"Please Enter conduction coefficient k    [W/m.K]  : "; cin>>k;
    cout<<"Please Enter convection coefficient h    [W/m^2.K]: "; cin>>h;
    cout<<"Please Enter Base Temperature ..... Tbas [K]      : "; cin>>T_b;
    cout<<"Please Enter Ambient Temperature .. Tamb [K]      : "; cin>>T_be;
    cout<<"Please Enter the Distance ......... X    [m]      : ";    cin>>x;
  }
  void Report()
  {
    cout<<"-----   Results at Location X = "<<x<<" [m] -----"<<endl;
    cout<<"T = "<<T<<endl;
    cout<<"q = "<<q<<endl;
    cout<<"n = "<<Co<<endl;
  }
  void Calculate()
  {
    Pe();
    AC();
    te();
    M2();
    m2();
    heat();
    Temp();
    performance();
    Report();
  }
  virtual void Pe()
  {
    P=(2*t+2*w);
  }
  virtual void AC()
  {
    Ac=w*t;
  }
  virtual void te()
  {
    teta_b=abs(T_b-T_be);
  }
  virtual void M2()
  {
    M=sqrt(h*P*k*Ac*teta_b);
  }
  virtual void m2()
  {
    m=sqrt((h*P)/(k*Ac));
  }
  virtual void performance()
  {
    Co=(q/(h*P*L*teta_b));
  }
};

//////////////convection//////////
class convection: public Fin
{
public:
  virtual void Temp()
  {
    T=(teta_b)*((cosh(m*(L-x))+(h/(m*k))*sinh(m*(L-x)))/(cosh(m*L)+(h/(m*k))*sinh(m*L)))+T_be;
  }
  virtual void heat()
  {
    q=M*((sinh(m*L)+(h/(m*k))*cosh(m*L))/(cosh(m*L)+(h/(m*k))*sinh(m*L)));
  }
};

//////////////adiabatic///////////
class adiabatic: public Fin
{
public:
  virtual void Temp() 
  {
    T=(teta_b)*(cosh(m*(L-x))/cosh(m*L))+T_be;
  }
  virtual void heat()
  {
    q=M*(tanh(m*L));
  }
};

//////prescribed temperature//////
class prescribedT: public Fin
{
public:
  virtual void Temp() 
  {
    T=(teta_b)*((((abs(T_L-T_be))/teta_b)*sinh(m*x)+sinh(m*(L-x)))/sinh(m*L))+T_be;
  }
  virtual void heat()
  {
    q=M*((cosh(m*L)-((abs(T_L-T_be))/teta_b))/sinh(m*L));
  }
};

/////////////infinate/////////////
class infinite: public Fin
{
public:
  virtual void Temp() 
  {
    T=(teta_b)*(exp((-1)*m*x))+T_be;
  }
  virtual void heat()
  {
    q=M;
  }
};

int main()
{
  Fin *fin;
  int c;
  cout<<"in the name of GOD \n";
  cout<<"uniform cross section program\n";
  cout<<"program only for rectangle section \n"; 
  cout<<"----------------------------------------------- \n"; 
  cout<<"[1]Convection [2]Adiabatic [3]Prescribed Tempereture [4]Infinite \n";
  cout<<"please select your Fin type: "; 
  cin>>c;
  
  switch (c)
  {
  case (1):
    {
      fin=new convection;
      break;
    }
  case (2):
    {
      fin=new adiabatic;
      break;
    }
  case (3):
    {
      fin=new prescribedT;
      cout<<"Please Enter the Tip Temperature (at X=L): ";
      cin>>fin->T_L;
      break;
    }
  case (4):
    {
      fin=new infinite;
      break;
    }
  default :
    {
      cout<<"the number you select is wrong";
    }
  }
  fin->ReadData();
  fin->Calculate();
  int i;
  cin>>i;
  return 0;
}